#import "MetrixPlugin.h"

@implementation MetrixPlugin
FlutterResult userIdResult;
FlutterResult attributionResult;

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel
                                     methodChannelWithName:@"metrix"
                                     binaryMessenger:[registrar messenger]];
    MetrixPlugin* instance = [[MetrixPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([@"initialize" isEqualToString:call.method]) {
        
        NSString *yourAppId = call.arguments[@"appId"];
        NSString *environment = MXEnvironmentProduction;
        MXConfig *metrixConfig = [MXConfig configWithAppId:yourAppId
                                               environment:environment];
        [metrixConfig setDelegate:self];
        
        [Metrix appDidLaunch:metrixConfig];
        result(nil);
    } else if([@"newEvent" isEqualToString:call.method]) {
        NSString *slug = call.arguments[@"slug"];
        
        NSDictionary *myAttributes = [[NSMutableDictionary alloc] init];
        NSDictionary *myMetrics =[[NSMutableDictionary alloc] init];
        if (call.arguments[@"attributes"]) {
            myAttributes = call.arguments[@"attributes"];
        }
        if (call.arguments[@"metrics"]) {
            myMetrics = call.arguments[@"metrics"];
        }
        
        MXCustomEvent *event = [MXCustomEvent newEvent:slug attributes:myAttributes metrics:myMetrics];
        [Metrix trackCustomEvent:event];
        result(nil);
    } else if([@"setAttributionMethod" isEqualToString:call.method]){
        userIdResult = result;
    }else if ([@"setUserIdMethod" isEqualToString:call.method]){
        attributionResult = result;
    }

    else {
        result(FlutterMethodNotImplemented);
    }
}


- (void)metrixSessionTrackingSucceeded:(MXSessionSuccess *)sessionSuccessResponseData {
    if(userIdResult != nil){
        userIdResult([sessionSuccessResponseData mxid]);
    }
 }


- (void)metrixAttributionChanged:(MXAttribution *)attribution {
    if(attributionResult != nil){
        NSError *error;
        NSData *jsonData = [NSJSONSerialization dataWithJSONObject:[attribution dictionary]
                                                           options:NSJSONWritingPrettyPrinted
                                                             error:&error];
        NSString *jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];

        attributionResult(jsonString);
    }
 }

@end
